Package: CollocInfer
Version: 1.0.5
Date: 2024-11-04
Title: Collocation Inference for Dynamic Systems
Authors@R: c(person(given="Giles",family="Hooker",email="ghooker@wharton.upenn.edu",role=c("aut","cre")),
         person(given="Luo",family="Xiao",email="lxiao5@ncsu.edu",role="aut"),
         person(given="James",family="Ramsay",email="ramsay@psych.mcgill.ca", role="ctb"))
Maintainer: Giles Hooker <ghooker@wharton.upenn.edu>
Depends: R (>= 4.3.0), fda
Imports: MASS, Matrix, spam, deSolve, methods
Suggests: pomp, SparseM, subplex, trust, maxLik
Description: These functions implement collocation-inference
    for continuous-time and discrete-time stochastic processes.
    They provide model-based smoothing, gradient-matching,
    generalized profiling and forwards prediction error methods.
License: GPL (>= 2)
URL: http://www.gileshooker.com
LazyData: true
NeedsCompilation: no
Packaged: 2024-11-05 04:08:20 UTC; giles
Author: Giles Hooker [aut, cre],
  Luo Xiao [aut],
  James Ramsay [ctb]
Repository: CRAN
Date/Publication: 2024-11-05 08:50:01 UTC
Built: R 4.6.0; ; 2026-04-30 09:13:54 UTC; unix
